@@ -1,11 +1,12 @@
 //! Bluetooth adapter.
 
 use dbus::{
-    arg::{RefArg, Variant},
+    arg::{PropMap, RefArg, Variant},
     nonblock::{Proxy, SyncConnection},
     Path,
 };
 use futures::{
+    future,
     stream::{self, SelectAll},
     Stream, StreamExt,
 };
@@ -13,7 +14,6 @@ use std::{
     collections::{BTreeSet, HashMap, HashSet},
     fmt::{Debug, Formatter},
     sync::Arc,
-    u32,
 };
 use strum::{Display, EnumString};
 use tokio::sync::mpsc;
@@ -25,7 +25,9 @@ use crate::{
     adv::{Advertisement, AdvertisementHandle, Capabilities, Feature, PlatformFeature, SecondaryChannel},
     all_dbus_objects, device,
     device::Device,
-    gatt, Address, AddressType, Error, ErrorKind, Event, InternalErrorKind, Modalias, Result, SessionInner,
+    gatt,
+    monitor::MonitorManager,
+    Address, AddressType, Error, ErrorKind, Event, InternalErrorKind, Modalias, Result, SessionInner,
     SingleSessionToken, SERVICE_NAME, TIMEOUT,
 };
 
@@ -47,7 +49,7 @@ pub struct Adapter {
 
 impl Debug for Adapter {
     fn fmt(&self, f: &mut Formatter) -> std::fmt::Result {
-        write!(f, "Adapter {{ name: {} }}", self.name())
+        f.debug_struct("Adapter").field("name", &self.name()).finish()
     }
 }
 
@@ -67,7 +69,7 @@ impl Adapter {
     }
 
     pub(crate) fn dbus_path(adapter_name: &str) -> Result<Path<'static>> {
-        Path::new(format!("{}{}", PREFIX, adapter_name,))
+        Path::new(format!("{PREFIX}{adapter_name}",))
             .map_err(|_| Error::new(ErrorKind::InvalidName((*adapter_name).to_string())))
     }
 
@@ -98,7 +100,7 @@ impl Adapter {
     /// Bluetooth addresses of discovered Bluetooth devices.
     pub async fn device_addresses(&self) -> Result<Vec<Address>> {
         let mut addrs = Vec::new();
-        for (path, interfaces) in all_dbus_objects(&*self.inner.connection).await? {
+        for (path, interfaces) in all_dbus_objects(&self.inner.connection).await? {
             match Device::parse_dbus_path(&path) {
                 Some((adapter, addr)) if adapter == *self.name && interfaces.contains_key(device::INTERFACE) => {
                     addrs.push(addr)
@@ -109,11 +111,45 @@ impl Adapter {
         Ok(addrs)
     }
 
+    /// Starts monitoring of advertisements.
+    ///
+    /// Once a monitoring job is activated by BlueZ, the client can expect to get
+    /// notified on the targeted advertisements no matter if there is an ongoing
+    /// discovery session.
+    ///
+    /// Use the returned [`MonitorManager`] to target advertisements
+    /// and drop it to stop monitoring advertisements.
+    pub async fn monitor(&self) -> Result<MonitorManager> {
+        MonitorManager::new(self.inner.clone(), self.name()).await
+    }
+
     /// Get interface to Bluetooth device of specified address.
     pub fn device(&self, address: Address) -> Result<Device> {
         Device::new(self.inner.clone(), self.name.clone(), address)
     }
 
+    /// Gets the filter used for device discovery.
+    pub async fn discovery_filter(&self) -> DiscoveryFilter {
+        self.inner.adapter_discovery_filter.lock().await.get(self.name()).cloned().unwrap_or_default()
+    }
+
+    /// Sets the filter used for device discovery.
+    ///
+    /// Setting a discovery filter does not guarantee that all its filters will be applied.
+    /// A discovery session from another program might be active, leading to merging of
+    /// the discovery filters by the Bluetooth daemon.
+    ///
+    /// The discovery filter can only be changed when no device discovery is currently active.
+    /// Otherwise a [DiscoveryActive error](ErrorKind::DiscoveryActive) will be returned.
+    pub async fn set_discovery_filter(&self, discovery_filter: DiscoveryFilter) -> Result<()> {
+        if self.inner.is_single_session_active(&self.dbus_path).await {
+            return Err(Error::new(ErrorKind::DiscoveryActive));
+        }
+
+        self.inner.adapter_discovery_filter.lock().await.insert(self.name().to_string(), discovery_filter);
+        Ok(())
+    }
+
     /// This method starts the device discovery session.
     ///
     /// This includes an inquiry procedure and remote device name resolving.
@@ -129,12 +165,20 @@ impl Adapter {
     /// yet when a [DeviceAdded event](AdapterEvent::DeviceAdded) occurs.
     /// Use [discover_devices_with_changes](Self::discover_devices_with_changes)
     /// when you want to be notified when the device properties change.
+    ///
+    /// The discovery filter can be configured using [set_discovery_filter](Self::set_discovery_filter).
     pub async fn discover_devices(&self) -> Result<impl Stream<Item = AdapterEvent>> {
         let token = self.discovery_session().await?;
-        let change_events = self.events().await?.map(move |evt| {
-            let _token = &token;
-            evt
-        });
+        let change_events = self
+            .events()
+            .await?
+            .map(move |evt| {
+                let _token = &token;
+                evt
+            })
+            .take_while(|evt| {
+                future::ready(!matches!(evt, AdapterEvent::PropertyChanged(AdapterProperty::Discovering(false))))
+            });
 
         let known = self.device_addresses().await?;
         let known_events = stream::iter(known).map(AdapterEvent::DeviceAdded);
@@ -155,6 +199,8 @@ impl Adapter {
     /// All already known devices are also included in the device stream.
     /// This may include devices that are currently not in range.
     /// Check the [Device::rssi] property to see if the device is currently present.
+    ///
+    /// The discovery filter can be configured using [set_discovery_filter](Self::set_discovery_filter).
     pub async fn discover_devices_with_changes(&self) -> Result<impl Stream<Item = AdapterEvent>> {
         let (tx, rx) = mpsc::channel(1);
         let mut discovery = self.discover_devices().await?;
@@ -196,17 +242,15 @@ impl Adapter {
     async fn discovery_session(&self) -> Result<SingleSessionToken> {
         let dbus_path = self.dbus_path.clone();
         let connection = self.inner.connection.clone();
-        self.inner
+        let token = self
+            .inner
             .single_session(
                 &self.dbus_path,
                 async move {
-                    let filter = DiscoveryFilter {
-                        duplicate_data: false,
-                        transport: DiscoveryTransport::Auto,
-                        ..Default::default()
-                    };
-                    self.call_method("SetDiscoveryFilter", (filter.into_dict(),)).await?;
-                    self.call_method("StartDiscovery", ()).await?;
+                    let () = self
+                        .call_method("SetDiscoveryFilter", (self.discovery_filter().await.into_dict(),))
+                        .await?;
+                    let () = self.call_method("StartDiscovery", ()).await?;
                     Ok(())
                 },
                 async move {
@@ -217,7 +261,8 @@ impl Adapter {
                     log::trace!("{}: {}.StopDiscovery () -> {:?}", &dbus_path, SERVICE_NAME, &result);
                 },
             )
-            .await
+            .await?;
+        Ok(token)
     }
 
     dbus_interface!();
@@ -309,7 +354,7 @@ impl Adapter {
     /// It will remove also the pairing information.
     pub async fn remove_device(&self, address: Address) -> Result<()> {
         let path = Device::dbus_path(self.name(), address)?;
-        self.call_method("RemoveDevice", ((path),)).await?;
+        let () = self.call_method("RemoveDevice", ((path),)).await?;
         Ok(())
     }
 
@@ -338,11 +383,11 @@ impl Adapter {
     ///
     /// This method is experimental.
     pub async fn connect_device(&self, address: Address, address_type: AddressType) -> Result<Device> {
-        let mut m = HashMap::new();
-        m.insert("Address", address.to_string());
+        let mut m = PropMap::new();
+        m.insert("Address".to_string(), Variant(address.to_string().box_clone()));
         match address_type {
             AddressType::LePublic | AddressType::LeRandom => {
-                m.insert("AddressType", address_type.to_string());
+                m.insert("AddressType".to_string(), Variant(address_type.to_string().box_clone()));
             }
             AddressType::BrEdr => (),
         }
@@ -581,7 +626,7 @@ define_properties!(
         /// corresponding PHY.
         property(
             SupportedAdvertisingSecondaryChannels, BTreeSet<SecondaryChannel>,
-            dbus: (adv::MANAGER_INTERFACE, "SupportedSecondaryChannels", Vec<String>, MANDATORY),
+            dbus: (adv::MANAGER_INTERFACE, "SupportedSecondaryChannels", Vec<String>, OPTIONAL),
             get: (supported_advertising_secondary_channels, v => {
                 v.iter().filter_map(|s| s.parse().ok()).collect()
             }),
@@ -615,6 +660,7 @@ define_properties!(
 /// Bluetooth adapter event.
 #[cfg_attr(docsrs, doc(cfg(feature = "bluetoothd")))]
 #[derive(Clone, Debug)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub enum AdapterEvent {
     /// Bluetooth device with specified address was added.
     DeviceAdded(Address),
@@ -625,8 +671,11 @@ pub enum AdapterEvent {
 }
 
 /// Transport parameter determines the type of scan.
+#[cfg_attr(docsrs, doc(cfg(feature = "bluetoothd")))]
 #[derive(Debug, Clone, Copy, Eq, PartialEq, Display, EnumString)]
-pub(crate) enum DiscoveryTransport {
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+#[non_exhaustive]
+pub enum DiscoveryTransport {
     /// interleaved scan
     #[strum(serialize = "auto")]
     Auto,
@@ -645,8 +694,13 @@ impl Default for DiscoveryTransport {
 }
 
 /// Bluetooth device discovery filter.
-#[derive(Clone, Debug, Eq, PartialEq)]
-pub(crate) struct DiscoveryFilter {
+///
+/// The default discovery filter does not restrict any devices and provides
+/// [duplicate data](Self::duplicate_data).
+#[cfg_attr(docsrs, doc(cfg(feature = "bluetoothd")))]
+#[derive(Default, Clone, Debug, Eq, PartialEq)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+pub struct DiscoveryFilter {
     ///  Filter by service UUIDs, empty means match
     ///  _any_ UUID.
     ///
@@ -673,6 +727,8 @@ pub(crate) struct DiscoveryFilter {
     /// PropertiesChanged signals will be emitted
     /// for already existing Device objects, with
     /// updated Pathloss value.
+    ///
+    /// Must not be set when [`rssi`](Self::rssi) is set.
     pub pathloss: Option<u16>,
     /// Transport parameter determines the type of
     /// scan.
@@ -714,26 +770,16 @@ pub(crate) struct DiscoveryFilter {
     /// it work as a logical OR, also setting empty
     /// string "" pattern will match any device found.
     pub pattern: Option<String>,
-}
-
-impl Default for DiscoveryFilter {
-    fn default() -> Self {
-        Self {
-            uuids: Default::default(),
-            rssi: Default::default(),
-            pathloss: Default::default(),
-            transport: Default::default(),
-            duplicate_data: true,
-            discoverable: false,
-            pattern: Default::default(),
-        }
-    }
+    #[doc(hidden)]
+    #[cfg_attr(feature = "serde", serde(skip))]
+    pub _non_exhaustive: (),
 }
 
 impl DiscoveryFilter {
     fn into_dict(self) -> HashMap<&'static str, Variant<Box<dyn RefArg>>> {
         let mut hm: HashMap<&'static str, Variant<Box<dyn RefArg>>> = HashMap::new();
-        let Self { uuids, rssi, pathloss, transport, duplicate_data, discoverable, pattern } = self;
+        let Self { uuids, rssi, pathloss, transport, duplicate_data, discoverable, pattern, _non_exhaustive } =
+            self;
         hm.insert("UUIDs", Variant(Box::new(uuids.into_iter().map(|uuid| uuid.to_string()).collect::<Vec<_>>())));
         if let Some(rssi) = rssi {
             hm.insert("RSSI", Variant(Box::new(rssi)));
@@ -742,8 +788,18 @@ impl DiscoveryFilter {
             hm.insert("Pathloss", Variant(Box::new(pathloss)));
         }
         hm.insert("Transport", Variant(Box::new(transport.to_string())));
-        hm.insert("DuplicateData", Variant(Box::new(duplicate_data)));
-        hm.insert("Discoverable", Variant(Box::new(discoverable)));
+
+        // WORKAROUND: bluetoothd has a bug that causes it to crash when deserializing a bool
+        //             from D-Bus on some architectures (at least 32-bit ARM). We can partly
+        //             work around this by omitting the bools from the dictionary, if they match
+        //             the default values.
+        if duplicate_data {
+            hm.insert("DuplicateData", Variant(Box::new(true)));
+        }
+        if discoverable {
+            hm.insert("Discoverable", Variant(Box::new(true)));
+        }
+
         if let Some(pattern) = pattern {
             hm.insert("Pattern", Variant(Box::new(pattern)));
         }
