@@ -23,6 +23,7 @@ pub(crate) const ADVERTISEMENT_PREFIX: &str = publish_path!("advertising/");
 
 /// Determines the type of advertising packet requested.
 #[derive(Clone, Copy, Debug, Eq, PartialEq, PartialOrd, Ord, Hash, Display, EnumString)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub enum Type {
     /// Broadcast
     #[strum(serialize = "broadcast")]
@@ -40,6 +41,7 @@ impl Default for Type {
 
 /// Secondary channel for advertisement.
 #[derive(Clone, Copy, Debug, Eq, PartialEq, PartialOrd, Ord, Hash, Display, EnumString)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 #[non_exhaustive]
 pub enum SecondaryChannel {
     /// 1M
@@ -61,6 +63,7 @@ impl Default for SecondaryChannel {
 
 /// Advertisement feature.
 #[derive(Clone, Copy, Debug, Eq, PartialEq, PartialOrd, Ord, Hash, Display, EnumString)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 #[non_exhaustive]
 pub enum Feature {
     /// TX power.
@@ -76,6 +79,7 @@ pub enum Feature {
 
 /// LE advertising platform feature.
 #[derive(Clone, Copy, Debug, Eq, PartialEq, PartialOrd, Ord, Hash, Display, EnumString)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 #[non_exhaustive]
 pub enum PlatformFeature {
     /// Indicates whether platform can
@@ -92,6 +96,7 @@ pub enum PlatformFeature {
 
 /// Advertising-related controller capabilities.
 #[derive(Clone, Debug, Default, Eq, PartialEq, PartialOrd, Ord, Hash)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 #[non_exhaustive]
 pub struct Capabilities {
     /// Maximum advertising data length.
@@ -125,6 +130,8 @@ impl Capabilities {
 ///
 /// Use [Adapter::advertise] to register a new advertisement.
 #[derive(Clone, Debug, Default, Eq, PartialEq, PartialOrd, Ord, Hash)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+#[cfg_attr(feature = "serde", serde(default))]
 pub struct Advertisement {
     /// Determines the type of advertising packet requested.
     pub advertisement_type: Type,
@@ -153,14 +160,18 @@ pub struct Advertisement {
     ///
     /// Note: Types already handled by other properties shall
     /// not be used.
-    pub advertisting_data: BTreeMap<u8, Vec<u8>>,
+    pub advertising_data: BTreeMap<u8, Vec<u8>>,
     /// Advertise as general discoverable.
     ///
     /// When present this
     /// will override adapter Discoverable property.
     ///
     /// Note: This property shall not be set when Type is set
-    /// to broadcast.
+    /// to broadcast. Additionally, Types that are official
+    /// Bluetooth assigned numbers cannot be used. So for
+    /// example the Type value of 0x0a cannot be used because
+    /// it is assigned as the TX Power Level Type. But,
+    /// currently the Type 0x0c is unassigned and can be used.
     pub discoverable: Option<bool>,
     /// The discoverable timeout in seconds.
     ///
@@ -244,7 +255,7 @@ impl Advertisement {
                 Some(la.service_data.iter().map(|(k, v)| (k.to_string(), Variant(v.clone()))).collect::<HashMap<_, _>>())
             });
             cr_property!(ib, "Data", la => {
-                Some(la.advertisting_data.clone().into_iter().collect::<HashMap<_, _>>())
+                Some(la.advertising_data.iter().map(|(k, v)| (*k, Variant(v.clone()))).collect::<HashMap<_, _>>())
             });
             cr_property!(ib, "Discoverable", la => {
                 la.discoverable
@@ -285,7 +296,7 @@ impl Advertisement {
     pub(crate) async fn register(
         self, inner: Arc<SessionInner>, adapter_name: Arc<String>,
     ) -> Result<AdvertisementHandle> {
-        let name = dbus::Path::new(format!("{}{}", ADVERTISEMENT_PREFIX, Uuid::new_v4().to_simple())).unwrap();
+        let name = dbus::Path::new(format!("{}{}", ADVERTISEMENT_PREFIX, Uuid::new_v4().as_simple())).unwrap();
         log::trace!("Publishing advertisement at {}", &name);
 
         {
@@ -295,8 +306,9 @@ impl Advertisement {
 
         log::trace!("Registering advertisement at {}", &name);
         let proxy =
-            Proxy::new(SERVICE_NAME, Adapter::dbus_path(&*adapter_name)?, TIMEOUT, inner.connection.clone());
-        proxy.method_call(MANAGER_INTERFACE, "RegisterAdvertisement", (name.clone(), PropMap::new())).await?;
+            Proxy::new(SERVICE_NAME, Adapter::dbus_path(&adapter_name)?, TIMEOUT, inner.connection.clone());
+        let () =
+            proxy.method_call(MANAGER_INTERFACE, "RegisterAdvertisement", (name.clone(), PropMap::new())).await?;
 
         let (drop_tx, drop_rx) = oneshot::channel();
         let unreg_name = name.clone();
@@ -319,6 +331,7 @@ impl Advertisement {
 /// Handle to active Bluetooth LE advertisement.
 ///
 /// Drop to unregister advertisement.
+#[must_use = "AdvertisementHandle must be held for advertisement to be broadcasted"]
 pub struct AdvertisementHandle {
     name: dbus::Path<'static>,
     _drop_tx: oneshot::Sender<()>,
