@@ -20,6 +20,7 @@ pub(crate) const AGENT_PREFIX: &str = publish_path!("agent/");
 
 /// Error response from us to a Bluetooth agent request.
 #[derive(Clone, Copy, Debug, displaydoc::Display, Eq, PartialEq, Ord, PartialOrd, Hash, IntoStaticStr)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 #[non_exhaustive]
 pub enum ReqError {
     /// Request was rejected.
@@ -427,7 +428,7 @@ impl RegisteredAgent {
                 },
             );
             ib.method_with_cr_async(
-                "RequestConfirmation",
+                "AuthorizeService",
                 ("device", "uuid"),
                 (),
                 |ctx, cr, (device, uuid): (dbus::Path<'static>, String)| {
@@ -453,7 +454,7 @@ impl RegisteredAgent {
     }
 
     pub(crate) async fn register(self, inner: Arc<SessionInner>) -> Result<AgentHandle> {
-        let name = dbus::Path::new(format!("{}{}", AGENT_PREFIX, Uuid::new_v4().to_simple())).unwrap();
+        let name = dbus::Path::new(format!("{}{}", AGENT_PREFIX, Uuid::new_v4().as_simple())).unwrap();
         let capability = self.a.capability();
         let request_default = self.a.request_default;
         log::trace!("Publishing agent at {} with capability {}", &name, &capability);
@@ -465,7 +466,7 @@ impl RegisteredAgent {
 
         log::trace!("Registering agent at {}", &name);
         let proxy = Proxy::new(SERVICE_NAME, MANAGER_PATH, TIMEOUT, inner.connection.clone());
-        proxy.method_call(MANAGER_INTERFACE, "RegisterAgent", (name.clone(), capability)).await?;
+        let () = proxy.method_call(MANAGER_INTERFACE, "RegisterAgent", (name.clone(), capability)).await?;
         let connection = inner.connection.clone();
 
         let (drop_tx, drop_rx) = oneshot::channel();
@@ -485,7 +486,7 @@ impl RegisteredAgent {
         if request_default {
             log::trace!("Requesting default agent for {}", &name);
             let proxy = Proxy::new(SERVICE_NAME, MANAGER_PATH, TIMEOUT, connection);
-            proxy.method_call(MANAGER_INTERFACE, "RequestDefaultAgent", (name.clone(),)).await?;
+            let () = proxy.method_call(MANAGER_INTERFACE, "RequestDefaultAgent", (name.clone(),)).await?;
         }
 
         Ok(AgentHandle { name, _drop_tx: drop_tx })
@@ -495,6 +496,7 @@ impl RegisteredAgent {
 /// Handle to registered agent.
 ///
 /// Drop to unregister agent.
+#[must_use = "AgentHandle must be held for agent to be registered"]
 pub struct AgentHandle {
     name: dbus::Path<'static>,
     _drop_tx: oneshot::Sender<()>,
