@@ -7,12 +7,12 @@ use dbus::{
 };
 use futures::{Stream, StreamExt};
 use std::{fmt, os::unix::prelude::FromRawFd, sync::Arc};
-use tokio::net::UnixStream;
+use tokio::net::UnixDatagram;
 use uuid::Uuid;
 
 use super::{
-    CharacteristicFlags, CharacteristicReader, CharacteristicWriter, WriteOp, CHARACTERISTIC_INTERFACE,
-    DESCRIPTOR_INTERFACE, SERVICE_INTERFACE,
+    mtu_workaround, CharacteristicFlags, CharacteristicReader, CharacteristicWriter, WriteOp,
+    CHARACTERISTIC_INTERFACE, DESCRIPTOR_INTERFACE, SERVICE_INTERFACE,
 };
 use crate::{
     all_dbus_objects, Address, Device, Error, ErrorKind, Event, InternalErrorKind, Result, SessionInner,
@@ -51,7 +51,7 @@ impl Service {
     ) -> Result<Self> {
         Ok(Self {
             inner,
-            dbus_path: Self::dbus_path(&*adapter_name, device_address, id)?,
+            dbus_path: Self::dbus_path(&adapter_name, device_address, id)?,
             adapter_name,
             device_address,
             id,
@@ -64,7 +64,7 @@ impl Service {
 
     pub(crate) fn dbus_path(adapter_name: &str, device_address: Address, id: u16) -> Result<Path<'static>> {
         let device_path = Device::dbus_path(adapter_name, device_address)?;
-        Ok(Path::new(format!("{}/service{:04x}", device_path, id)).unwrap())
+        Ok(Path::new(format!("{device_path}/service{id:04x}")).unwrap())
     }
 
     pub(crate) fn parse_dbus_path_prefix<'a>(path: &'a Path) -> Option<((&'a str, Address, u16), &'a str)> {
@@ -110,7 +110,7 @@ impl Service {
     /// GATT characteristics belonging to this service.
     pub async fn characteristics(&self) -> Result<Vec<Characteristic>> {
         let mut chars = Vec::new();
-        for (path, interfaces) in all_dbus_objects(&*self.inner.connection).await? {
+        for (path, interfaces) in all_dbus_objects(&self.inner.connection).await? {
             match Characteristic::parse_dbus_path(&path) {
                 Some((adapter, device_address, service_id, id))
                     if adapter == *self.adapter_name
@@ -142,7 +142,9 @@ impl Service {
 }
 
 define_properties!(
-    Service, ServiceProperty => {
+    Service,
+    /// GATT service property.
+    pub ServiceProperty => {
         /// Indicates whether or not this GATT service is a
         /// primary service.
         ///
@@ -205,7 +207,7 @@ impl Characteristic {
     ) -> Result<Self> {
         Ok(Self {
             inner,
-            dbus_path: Self::dbus_path(&*adapter_name, device_address, service_id, id)?,
+            dbus_path: Self::dbus_path(&adapter_name, device_address, service_id, id)?,
             adapter_name,
             device_address,
             service_id,
@@ -221,7 +223,7 @@ impl Characteristic {
         adapter_name: &str, device_address: Address, service_id: u16, id: u16,
     ) -> Result<Path<'static>> {
         let service_path = Service::dbus_path(adapter_name, device_address, service_id)?;
-        Ok(Path::new(format!("{}/char{:04x}", service_path, id)).unwrap())
+        Ok(Path::new(format!("{service_path}/char{id:04x}")).unwrap())
     }
 
     #[allow(clippy::type_complexity)]
@@ -273,7 +275,7 @@ impl Characteristic {
     /// GATT descriptors belonging to this characteristic.
     pub async fn descriptors(&self) -> Result<Vec<Descriptor>> {
         let mut chars = Vec::new();
-        for (path, interfaces) in all_dbus_objects(&*self.inner.connection).await? {
+        for (path, interfaces) in all_dbus_objects(&self.inner.connection).await? {
             match Descriptor::parse_dbus_path(&path) {
                 Some((adapter, device_address, service_id, char_id, id))
                     if adapter == *self.adapter_name
@@ -349,12 +351,16 @@ impl Characteristic {
     pub async fn write_io(&self) -> Result<CharacteristicWriter> {
         let options = PropMap::new();
         let (fd, mtu): (OwnedFd, u16) = self.call_method("AcquireWrite", (options,)).await?;
-        let stream = unsafe { std::os::unix::net::UnixStream::from_raw_fd(fd.into_fd()) };
-        stream.set_nonblocking(true)?;
-        let stream = UnixStream::from_std(stream)?;
-        // WORKAROUND: BlueZ drops data at end of packet if full MTU is used.
-        let mtu = mtu.saturating_sub(5).into();
-        Ok(CharacteristicWriter { mtu, stream })
+        let socket = unsafe { std::os::unix::net::UnixDatagram::from_raw_fd(fd.into_fd()) };
+        socket.set_nonblocking(true)?;
+        let socket = UnixDatagram::from_std(socket)?;
+        let mtu = mtu_workaround(mtu.into());
+        Ok(CharacteristicWriter {
+            adapter_name: self.adapter_name().to_string(),
+            device_address: self.device_address,
+            mtu,
+            socket,
+        })
     }
 
     /// Starts a notification or indication session from this characteristic
@@ -387,7 +393,7 @@ impl Characteristic {
             .single_session(
                 &self.dbus_path,
                 async move {
-                    self.call_method("StartNotify", ()).await?;
+                    let () = self.call_method("StartNotify", ()).await?;
                     Ok(())
                 },
                 async move {
@@ -425,10 +431,16 @@ impl Characteristic {
     pub async fn notify_io(&self) -> Result<CharacteristicReader> {
         let options = PropMap::new();
         let (fd, mtu): (OwnedFd, u16) = self.call_method("AcquireNotify", (options,)).await?;
-        let stream = unsafe { std::os::unix::net::UnixStream::from_raw_fd(fd.into_fd()) };
-        stream.set_nonblocking(true)?;
-        let stream = UnixStream::from_std(stream)?;
-        Ok(CharacteristicReader { mtu: mtu.into(), stream, buf: Vec::new() })
+        let socket = unsafe { std::os::unix::net::UnixDatagram::from_raw_fd(fd.into_fd()) };
+        socket.set_nonblocking(true)?;
+        let socket = UnixDatagram::from_std(socket)?;
+        Ok(CharacteristicReader {
+            adapter_name: self.adapter_name().to_string(),
+            device_address: self.device_address,
+            mtu: mtu.into(),
+            socket,
+            buf: Vec::new(),
+        })
     }
 
     dbus_interface!();
@@ -476,7 +488,9 @@ impl CharacteristicWriteRequest {
 }
 
 define_properties!(
-    Characteristic, CharacteristicProperty => {
+    Characteristic,
+    /// GATT characteristic property.
+    pub CharacteristicProperty => {
         /// 128-bit characteristic UUID.
         property(
             Uuid, Uuid,
@@ -484,11 +498,14 @@ define_properties!(
             get: (uuid, v => {v.parse().map_err(|_| Error::new(ErrorKind::Internal(InternalErrorKind::InvalidUuid(v.to_string()))))?}),
         );
 
-        ///	True, if notifications or indications on this
-        ///	characteristic are currently enabled.
+        ///	Whether notifications or indications on this
+        ///	characteristic are supported and currently enabled.
+        ///
+        /// Returns `Some(true)` if enabled and `Some(false)` if disabled.
+        /// When notifications are unsupported this returns `None`.
         property(
             Notifying, bool,
-            dbus: (CHARACTERISTIC_INTERFACE, "Notifying", bool, MANDATORY),
+            dbus: (CHARACTERISTIC_INTERFACE, "Notifying", bool, OPTIONAL),
             get: (notifying, v => {v.to_owned()}),
         );
 
@@ -514,6 +531,17 @@ define_properties!(
             dbus: (CHARACTERISTIC_INTERFACE, "Value", Vec<u8>, MANDATORY),
             get: (cached_value, v => {v.to_owned()}),
         );
+
+        /// The maximum transmission unit for the characteristic.
+        ///
+        /// This is the maximum amount of data that can be sent or received
+        /// in a single packet for this characteristic. Longer data may be
+        /// able to be sent or received using long procedures when available.
+        property(
+            Mtu, usize,
+            dbus: (CHARACTERISTIC_INTERFACE, "MTU", u16, MANDATORY),
+            get: (mtu, v => { mtu_workaround((*v).into()) }),
+        );
     }
 );
 
@@ -547,7 +575,7 @@ impl Descriptor {
     ) -> Result<Self> {
         Ok(Self {
             inner,
-            dbus_path: Self::dbus_path(&*adapter_name, device_address, service_id, characteristic_id, id)?,
+            dbus_path: Self::dbus_path(&adapter_name, device_address, service_id, characteristic_id, id)?,
             adapter_name,
             device_address,
             service_id,
@@ -564,7 +592,7 @@ impl Descriptor {
         adapter_name: &str, device_address: Address, service_id: u16, characteristic_id: u16, id: u16,
     ) -> Result<Path<'static>> {
         let char_path = Characteristic::dbus_path(adapter_name, device_address, service_id, characteristic_id)?;
-        Ok(Path::new(format!("{}/desc{:04x}", char_path, id)).unwrap())
+        Ok(Path::new(format!("{char_path}/desc{id:04x}")).unwrap())
     }
 
     #[allow(clippy::type_complexity)]
@@ -692,7 +720,9 @@ impl DescriptorWriteRequest {
 }
 
 define_properties!(
-    Descriptor, CharacteristicDescriptorProperty => {
+    Descriptor,
+    /// GATT characteristic descriptor property.
+    pub CharacteristicDescriptorProperty => {
         /// 128-bit descriptor UUID.
         property(
             Uuid, Uuid,
