@@ -79,6 +79,7 @@ pub const PSM_LE_MAX: u16 = 0xff;
 
 /// An L2CAP socket address.
 #[derive(Debug, Default, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct SocketAddr {
     /// Device address.
     ///
@@ -161,6 +162,7 @@ fn any_bind_addr(addr: &SocketAddr) -> SocketAddr {
 
 /// L2CAP socket security level.
 #[derive(Debug, Clone, Copy, Eq, PartialEq, PartialOrd, Ord, Hash, FromPrimitive, ToPrimitive)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub enum SecurityLevel {
     /// Insecure.
     Sdp = BT_SECURITY_SDP as _,
@@ -176,6 +178,7 @@ pub enum SecurityLevel {
 
 /// L2CAP socket security.
 #[derive(Debug, Clone, Copy, Eq, PartialEq, PartialOrd, Ord, Hash)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct Security {
     /// Level.
     pub level: SecurityLevel,
@@ -202,7 +205,14 @@ impl TryFrom<bt_security> for Security {
 
 /// L2CAP socket flow control mode.
 #[derive(Debug, Clone, Copy, Eq, PartialEq, PartialOrd, Ord, Hash, FromPrimitive, ToPrimitive)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub enum FlowControl {
+    /// Basic.
+    Basic = 0x00,
+    /// Enhanced retransmission mode (ERTM).
+    Ertm = 0x01,
+    /// Streaming.
+    Streaming = 0x02,
     /// LE flow control.
     Le = 0x03,
     /// Extended flow control.
@@ -744,6 +754,7 @@ impl AsyncWrite for Stream {
     }
 }
 
+#[allow(clippy::duplicate_mod)]
 #[path = "stream_util.rs"]
 pub mod stream;
 
