@@ -1,4 +1,8 @@
 #![cfg_attr(docsrs, feature(doc_cfg))]
+#![doc(
+    html_logo_url = "https://raw.githubusercontent.com/bluez/bluer/master/doc/bluez.png",
+    html_favicon_url = "https://raw.githubusercontent.com/bluez/bluer/master/doc/bluez.png"
+)]
 
 //! # BlueR — Official BlueZ Bindings for Rust
 //!
@@ -15,11 +19,12 @@
 //!     * configuration of power, discoverability, name, etc.
 //!     * hot-plug support through change events stream
 //! * [Bluetooth devices](Device)
-//!     * [discovery](Adapter::discover_devices)
+//!     * [discovery](Adapter::discover_devices) with custom filters
 //!     * querying of address, name, class, signal strength (RSSI), etc.
 //!     * Bluetooth Low Energy advertisements
 //!     * [change events stream](Adapter::events)
 //!     * connecting and pairing
+//!     * [passive LE advertisement monitoring](Adapter::monitor)
 //! * [consumption of remote GATT services](Device::services)
 //!     * GATT service discovery
 //!     * read, write and notify operations on characteristics
@@ -46,6 +51,9 @@
 //!     * support for classic Bluetooth (BR/EDR)
 //!     * stream oriented
 //!     * async IO interface with [AsyncRead] and [AsyncWrite] support
+//! * [Bluetooth Mesh](mesh)
+//!     * provision and join networks
+//!     * send and receive messages
 //! * [database of assigned numbers](id)
 //!     * manufacturer ids
 //!     * services classes, GATT services, characteristics and descriptors
@@ -54,12 +62,16 @@
 //! However, pull requests and contributions are welcome!
 //!
 //! ## Crate features
-//! All crate features are enabled by default.
+//! The following crate features are available.
 //!
 //! * `bluetoothd`: Enables all functions requiring a running Bluetooth daemon.
 //! * `id`: Enables database of assigned numbers.
 //! * `l2cap`: Enables L2CAP sockets.
 //! * `rfcomm`: Enables RFCOMM sockets.
+//! * `mesh`: Enables Bluetooth mesh functionality.
+//! * `serde`: Enables serialization and deserialization of some data types.
+//!
+//! To enable all crate features specify the `full` crate feature.
 //!
 //! ## Basic usage
 //! Create a [Session] using [Session::new]; this establishes a connection to the Bluetooth daemon.
@@ -77,6 +89,9 @@
 
 #![warn(missing_docs)]
 
+#[cfg(not(target_os = "linux"))]
+compile_error!("BlueR only supports the Linux operating system.");
+
 #[cfg(feature = "bluetoothd")]
 use dbus::{
     arg::{prop_cast, AppendAll, PropMap, RefArg, Variant},
@@ -89,6 +104,7 @@ use dbus_crossroads::{Context, Crossroads};
 use futures::Future;
 #[cfg(feature = "bluetoothd")]
 use hex::FromHex;
+use macaddr::MacAddr6;
 use num_derive::FromPrimitive;
 #[cfg(feature = "bluetoothd")]
 use std::{collections::HashMap, marker::PhantomData, sync::Arc, time::Duration};
@@ -266,6 +282,20 @@ macro_rules! define_properties {
         $dbus_interface:expr, $dbus_name:expr, $dbus_type:ty => $type:ty
     ) => {};
 
+    (@fetch_prop
+        $self:ident, $props_name:ident, $enum_name:ident, $name:ident, $getter_name:ident, OPTIONAL
+    ) => {
+        if let Some(value) = $self.$getter_name().await? {
+            $props_name.push($enum_name::$name(value));
+        }
+    };
+
+    (@fetch_prop
+        $self:ident, $props_name:ident, $enum_name:ident, $name:ident, $getter_name:ident, MANDATORY
+    ) => {
+        $props_name.push($enum_name::$name($self.$getter_name().await?));
+    };
+
     (
         $struct_name:ident, $(#[$enum_outer:meta])* $enum_vis:vis $enum_name:ident =>
         {$(
@@ -292,11 +322,27 @@ macro_rules! define_properties {
                     $dbus_interface, $dbus_name, $dbus_type => $type
                 );
             )*
+
+            /// Queries and returns all properties.
+            #[allow(dead_code)]
+            $enum_vis async fn all_properties(&self) -> Result<Vec<$enum_name>> {
+                let mut props = Vec::new();
+
+                $(
+                    define_properties!(@fetch_prop
+                        self, props, $enum_name, $name, $getter_name, $opt
+                    );
+                )*
+
+                Ok(props)
+            }
         }
 
         $(#[$enum_outer])*
         #[cfg_attr(docsrs, doc(cfg(feature = "bluetoothd")))]
         #[derive(Debug, Clone)]
+        #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+        #[non_exhaustive]
         $enum_vis enum $enum_name {
             $(
                 $(#[$outer])*
@@ -418,6 +464,7 @@ macro_rules! define_flags {
         )*
     }) => {
         #[derive(Clone, Copy, Default, Debug, Eq, PartialEq, Ord, PartialOrd, Hash)]
+        #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
         #[doc=$doc]
         $vis struct $name {
             $(
@@ -487,6 +534,12 @@ pub mod gatt;
 #[cfg(feature = "l2cap")]
 #[cfg_attr(docsrs, doc(cfg(feature = "l2cap")))]
 pub mod l2cap;
+#[cfg(feature = "mesh")]
+#[cfg_attr(docsrs, doc(cfg(feature = "mesh")))]
+pub mod mesh;
+#[cfg(feature = "bluetoothd")]
+#[cfg_attr(docsrs, doc(cfg(feature = "bluetoothd")))]
+pub mod monitor;
 #[cfg(feature = "rfcomm")]
 #[cfg_attr(docsrs, doc(cfg(feature = "rfcomm")))]
 pub mod rfcomm;
@@ -497,6 +550,7 @@ mod sys;
 #[cfg(feature = "bluetoothd")]
 pub use crate::{adapter::*, device::*, session::*};
 
+#[doc(no_inline)]
 pub use uuid::Uuid;
 mod uuid_ext;
 pub use uuid_ext::UuidExt;
@@ -509,6 +563,7 @@ pub mod id;
 #[cfg(feature = "bluetoothd")]
 #[cfg_attr(docsrs, doc(cfg(feature = "bluetoothd")))]
 #[derive(Clone, Debug, Eq, PartialEq, Ord, PartialOrd, Hash)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct Error {
     /// Error kind.
     pub kind: ErrorKind,
@@ -520,6 +575,8 @@ pub struct Error {
 #[cfg(feature = "bluetoothd")]
 #[cfg_attr(docsrs, doc(cfg(feature = "bluetoothd")))]
 #[derive(Clone, Debug, displaydoc::Display, Eq, PartialEq, Ord, PartialOrd, Hash, EnumString)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+#[ignore_extra_doc_attributes]
 #[non_exhaustive]
 pub enum ErrorKind {
     /// Bluetooth device already connected
@@ -579,6 +636,27 @@ pub enum ErrorKind {
     /// the target object was either not present or removed
     #[strum(disabled)]
     NotFound,
+    /// advertisement monitor could not be activated
+    #[strum(disabled)]
+    AdvertisementMonitorRejected,
+    /// the discovery filter cannot be changed while a discovery session is active
+    #[strum(disabled)]
+    DiscoveryActive,
+    /// joining the mesh network failed: {0}
+    #[cfg(feature = "mesh")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "mesh")))]
+    #[strum(disabled)]
+    MeshJoinFailed(mesh::application::JoinFailedReason),
+    /// adding a node to the mesh network failed: {0}
+    #[cfg(feature = "mesh")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "mesh")))]
+    #[strum(disabled)]
+    MeshAddNodeFailed(mesh::management::AddNodeFailedReason),
+    /// mesh element is not published
+    #[cfg(feature = "mesh")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "mesh")))]
+    #[strum(disabled)]
+    MeshElementUnpublished,
     /// internal error: {0}
     #[strum(disabled)]
     Internal(InternalErrorKind),
@@ -591,6 +669,7 @@ pub enum ErrorKind {
 #[cfg(feature = "bluetoothd")]
 #[cfg_attr(docsrs, doc(cfg(feature = "bluetoothd")))]
 #[derive(Clone, Debug, displaydoc::Display, Eq, PartialEq, Ord, PartialOrd, Hash)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 #[non_exhaustive]
 pub enum InternalErrorKind {
     /// invalid UUID: {0}
@@ -604,6 +683,8 @@ pub enum InternalErrorKind {
     /// join error
     JoinError,
     /// IO error {0:?}
+    // The error kind is not preserved during serialization.
+    #[cfg_attr(feature = "serde", serde(with = "io_errorkind_serde"))]
     Io(std::io::ErrorKind),
     /// D-Bus error {0}
     DBus(String),
@@ -679,12 +760,79 @@ impl From<InvalidAddress> for Error {
     }
 }
 
+#[cfg(feature = "bluetoothd")]
+impl From<Error> for std::io::Error {
+    fn from(err: Error) -> Self {
+        use std::io::ErrorKind as E;
+        let kind = match err.kind {
+            ErrorKind::AlreadyConnected => E::AlreadyExists,
+            ErrorKind::AlreadyExists => E::AlreadyExists,
+            ErrorKind::AuthenticationCanceled => E::PermissionDenied,
+            ErrorKind::AuthenticationFailed => E::PermissionDenied,
+            ErrorKind::AuthenticationRejected => E::PermissionDenied,
+            ErrorKind::AuthenticationTimeout => E::PermissionDenied,
+            ErrorKind::ConnectionAttemptFailed => E::ConnectionRefused,
+            ErrorKind::DoesNotExist => E::NotFound,
+            ErrorKind::Failed => E::Other,
+            ErrorKind::InProgress => E::Other,
+            ErrorKind::InvalidArguments => E::InvalidInput,
+            ErrorKind::InvalidLength => E::InvalidData,
+            ErrorKind::NotAvailable => E::NotFound,
+            ErrorKind::NotAuthorized => E::PermissionDenied,
+            ErrorKind::NotReady => E::Other,
+            ErrorKind::NotSupported => E::Unsupported,
+            ErrorKind::NotPermitted => E::PermissionDenied,
+            ErrorKind::InvalidOffset => E::InvalidInput,
+            ErrorKind::InvalidAddress(_) => E::InvalidInput,
+            ErrorKind::InvalidName(_) => E::InvalidInput,
+            ErrorKind::ServicesUnresolved => E::Other,
+            ErrorKind::NotRegistered => E::Other,
+            ErrorKind::NotificationSessionStopped => E::ConnectionReset,
+            ErrorKind::IndicationUnconfirmed => E::TimedOut,
+            ErrorKind::NotFound => E::NotFound,
+            ErrorKind::DiscoveryActive => E::PermissionDenied,
+            ErrorKind::AdvertisementMonitorRejected => E::InvalidInput,
+            #[cfg(feature = "mesh")]
+            ErrorKind::MeshJoinFailed(_) => E::ConnectionRefused,
+            #[cfg(feature = "mesh")]
+            ErrorKind::MeshAddNodeFailed(_) => E::ConnectionRefused,
+            #[cfg(feature = "mesh")]
+            ErrorKind::MeshElementUnpublished => E::InvalidInput,
+            ErrorKind::Internal(InternalErrorKind::Io(err)) => err,
+            ErrorKind::Internal(_) => E::Other,
+        };
+        std::io::Error::new(kind, err)
+    }
+}
+
+#[cfg(all(feature = "bluetoothd", feature = "serde"))]
+mod io_errorkind_serde {
+    pub fn serialize<S>(_kind: &std::io::ErrorKind, ser: S) -> Result<S::Ok, S::Error>
+    where
+        S: serde::Serializer,
+    {
+        use serde::Serialize;
+        ().serialize(ser)
+    }
+
+    pub fn deserialize<'de, D>(deser: D) -> Result<std::io::ErrorKind, D::Error>
+    where
+        D: serde::Deserializer<'de>,
+    {
+        use serde::Deserialize;
+        <()>::deserialize(deser)?;
+        Ok(std::io::ErrorKind::Other)
+    }
+}
+
 /// Bluetooth result.
 #[cfg(feature = "bluetoothd")]
 #[cfg_attr(docsrs, doc(cfg(feature = "bluetoothd")))]
 pub type Result<T> = std::result::Result<T, Error>;
 
 /// Bluetooth address.
+///
+/// The serialized representation is a string in colon-hexadecimal notation.
 #[derive(Clone, Copy, Default, Eq, PartialEq, Ord, PartialOrd, Hash)]
 pub struct Address(pub [u8; 6]);
 
@@ -728,7 +876,7 @@ impl Display for Address {
 
 impl Debug for Address {
     fn fmt(&self, f: &mut Formatter) -> fmt::Result {
-        write!(f, "{}", self)
+        write!(f, "{self}")
     }
 }
 
@@ -746,8 +894,21 @@ impl From<Address> for sys::bdaddr_t {
     }
 }
 
+impl From<MacAddr6> for Address {
+    fn from(addr: MacAddr6) -> Self {
+        Self(addr.into_array())
+    }
+}
+
+impl From<Address> for MacAddr6 {
+    fn from(addr: Address) -> Self {
+        addr.0.into()
+    }
+}
+
 /// Invalid Bluetooth address error.
 #[derive(Debug, Clone)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct InvalidAddress(pub String);
 
 impl fmt::Display for InvalidAddress {
@@ -781,8 +942,31 @@ impl From<Address> for [u8; 6] {
     }
 }
 
+#[cfg(feature = "serde")]
+impl serde::Serialize for Address {
+    fn serialize<S>(&self, serializer: S) -> std::result::Result<S::Ok, S::Error>
+    where
+        S: serde::Serializer,
+    {
+        self.to_string().serialize(serializer)
+    }
+}
+
+#[cfg(feature = "serde")]
+impl<'de> serde::Deserialize<'de> for Address {
+    fn deserialize<D>(deserializer: D) -> std::result::Result<Self, D::Error>
+    where
+        D: serde::Deserializer<'de>,
+    {
+        use serde::de::Error;
+        let s = String::deserialize(deserializer)?;
+        s.parse().map_err(D::Error::custom)
+    }
+}
+
 /// Bluetooth device address type.
 #[derive(Clone, Copy, Debug, Eq, PartialEq, Ord, PartialOrd, Hash, Display, EnumString, FromPrimitive)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 #[repr(u8)]
 pub enum AddressType {
     /// Classic Bluetooth (BR/EDR) address.
@@ -806,6 +990,7 @@ impl Default for AddressType {
 #[cfg(feature = "bluetoothd")]
 #[cfg_attr(docsrs, doc(cfg(feature = "bluetoothd")))]
 #[derive(Clone, Debug, Eq, PartialEq, Ord, PartialOrd, Hash)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct Modalias {
     /// Source.
     pub source: String,
@@ -825,6 +1010,7 @@ impl FromStr for Modalias {
         fn do_parse(m: &str) -> Option<Modalias> {
             let ids: Vec<&str> = m.split(':').collect();
 
+            #[allow(clippy::get_first)]
             let source = ids.get(0)?;
             let vendor = Vec::from_hex(ids.get(1)?.get(1..5)?).ok()?;
             let product = Vec::from_hex(ids.get(1)?.get(6..10)?).ok()?;
@@ -892,7 +1078,7 @@ fn method_call<
             let mut args = Vec::new();
             let mut arg_iter = ctx.message().iter_init();
             while let Some(value) = arg_iter.get_refarg() {
-                args.push(format!("{:?}", value));
+                args.push(format!("{value:?}"));
                 arg_iter.next();
             }
             log::trace!(
